#   Copyright (c) 2003-2007 Open Source Applications Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


import wx, webbrowser

from amazon import setLicense
from application import Utility, Globals
from i18n import MessageFactory, getLocaleSet
from osaf.startup import PeriodicTask

LICENSE_URL="https://aws-portal.amazon.com/gp/aws/developer/registration/index.html"
PLUGIN_NAME="Chandler-AmazonPlugin"
_ = MessageFactory(PLUGIN_NAME)

_SITE_CODES = ['us', 'gb', 'de', 'jp']
_SITE_LIST = [_(u"Amazon USA"),
              _(u"Amazon UK"),
              _(u"Amazon Germany"),
              _(u"Amazon Japan")]

_CAT_CODES = ['books', 'dvd', 'music']
_CAT_LIST = [_(u"Books"),
             _(u"DVD"),
             _(u"Music")]


def promptEmail():
    """
    Displays an Amazon Email Wishlist dialog. The dialog allows the user to
    choose the Amazon site (Amazon US, Amazon UK, Amazon Germany, Amazon
    Japan) which the Email wishlist is on.

    @rtype: tuple
    @return: tuple containing the Email Address to use and the country code
             for the Amazon Site or a tuple containing (None, None) if no
             results found. 
    """

    return _showDialog(_(u"New Amazon Wish List"),
                       _(u"Enter the Amazon email address of the wish list:"),
                       u"", False)

def promptKeywords():
    """
    Displays an Amazon Keyword search dialog. The dialog allows the user to
    choose the Amazon site (Amazon US, Amazon UK, Amazon Germany, Amazon
    Japan) and the category (Books, DVD, Music) which the keyword search is
    on.

    @rtype: tuple
    @return: tuple containing the search keywords to use, the country code
             for the Amazon Site, and the category to search on or a tuple
             containing (None, None, None) if no results found. 
    """

    return _showDialog(_(u"New Amazon Collection"),
                       _(u"Enter your Amazon search keywords:"),
                       u"Scott Rosenberg")


def _showDialog(title, message, value, showCategories=True):

    win = _promptAmazonDialog(wx.GetApp().mainFrame, -1, title, message,
                              value, showCategories)
    win.CenterOnScreen()
    val = win.ShowModal()

    if val == wx.ID_OK:
        # Assign the new values
        value = win.GetValue()

    elif showCategories:
        value = (None, None, None)
    else:
        value = (None, None)

    win.Destroy()

    return value


class _promptAmazonDialog(wx.Dialog):

    def __init__(self, parent, ID, title, message, value, showCategories):
        # Instead of calling wx.Dialog.__init__ we precreate the dialog
        # so we can set an extra style that must be set before
        # creation, and then we create the GUI dialog using the Create
        # method.
        pre = wx.PreDialog()
        pre.Create(parent, ID, title, wx.DefaultPosition, wx.DefaultSize,
                   wx.DEFAULT_DIALOG_STYLE)

        # This next step is the most important, it turns this Python
        # object into the real wrapper of the dialog (instead of pre)
        # as far as the wxPython extension is concerned.
        self.this = pre.this
        self.showCategories = showCategories

        # Now continue with the normal construction of the dialog
        # contents
        sizer = wx.BoxSizer(wx.VERTICAL)
        label = wx.StaticText(self, -1, message)
        sizer.Add(label, 0, wx.ALIGN_CENTER|wx.ALL, 5)

        box = wx.BoxSizer(wx.HORIZONTAL)
        text = wx.TextCtrl(self, -1, value, wx.DefaultPosition, [350,-1])
        box.Add(text, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        if self.showCategories:
            box = wx.BoxSizer(wx.HORIZONTAL)
            label = wx.StaticText(self, -1, _(u"Browse by category"))
            box.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 5)
            catChoice = wx.Choice(self, -1, choices=_CAT_LIST)
            box.Add(catChoice, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
            sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)
            catChoice.SetSelection(0)
            self.catChoiceControl = catChoice

        box = wx.BoxSizer(wx.HORIZONTAL)
        label = wx.StaticText(self, -1, _(u"Select a site to search"))
        box.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 5)
        siteChoice = wx.Choice(self, -1, choices=_SITE_LIST)
        box.Add(siteChoice, 1, wx.ALIGN_CENTRE|wx.ALL, 5)
        sizer.Add(box, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        box = wx.BoxSizer(wx.HORIZONTAL)
        btn = wx.Button(self, wx.ID_OK)
        btn.SetDefault()
        box.Add(btn, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        btn = wx.Button(self, wx.ID_CANCEL)
        box.Add(btn, 0, wx.ALIGN_CENTRE|wx.ALL, 5)

        sizer.Add(box, 0, wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

        # Store these, using attribute names that hopefully wont collide with
        # any wx attributes
        self.textControl = text
        self.siteChoiceControl = siteChoice

        siteChoice.SetSelection(self.GetSelectionPosition())

    def GetValue(self):
        if self.showCategories:
            return (self.textControl.GetValue(),
                    _SITE_CODES[self.siteChoiceControl.GetSelection()],
                    _CAT_CODES[self.catChoiceControl.GetSelection()])

        return (self.textControl.GetValue(),
                _SITE_CODES[self.siteChoiceControl.GetSelection()])


    def GetSelectionPosition(self):
        localeSet = getLocaleSet()

        for loc in localeSet:
            try:
                #raises a ValueError if the string does not contain a '_'
                loc.index("_")
                #Strip the country code from the locale string
                loc = loc.split("_")[1]
            except ValueError:
                pass

            for i in xrange(len(_SITE_CODES)):
                if _SITE_CODES[i] == loc.lower():
                    return i
        return 0


class LicenseTask(PeriodicTask):
    
    # target is periodic task
    def getTarget(self):
        return self

    # target is already constructed as self
    def __call__(self, periodicTask):
        return self

    # target needs no view of its own
    def fork(self):
        return self

    # target implementation
    def run(self):
        prefs = Utility.loadPrefs(Globals.options).get(PLUGIN_NAME)
        if prefs is not None:
            license = prefs.get('license')
            if license:
                setLicense(license)
        return False # no need to run again


class LicenseDialog(wx.Dialog):

    def __init__(self, parent, ID):

        # Instead of calling wx.Dialog.__init__ we precreate the dialog
        # so we can set an extra style that must be set before
        # creation, and then we create the GUI dialog using the Create
        # method.
        pre = wx.PreDialog()
        pre.Create(parent, ID, _(u"Enter Amazon Web Services License"),
                   wx.DefaultPosition, wx.DefaultSize, wx.DEFAULT_DIALOG_STYLE)

        # This next step is the most important, it turns this Python
        # object into the real wrapper of the dialog (instead of pre)
        # as far as the wxPython extension is concerned.
        self.this = pre.this

        # Now continue with the normal construction of the dialog
        # contents
        
        sizer = wx.BoxSizer(wx.VERTICAL)
        grid = wx.GridSizer(2, 2)

        # License (text control)....
        label = wx.StaticText(self, -1, _(u"License:"))
        grid.Add(label, 0, wx.ALIGN_LEFT|wx.ALL, 5)
        self.licenseText = wx.TextCtrl(self, -1, u"",
                                       wx.DefaultPosition, [150, -1])
        grid.Add(self.licenseText, 0, wx.ALIGN_CENTRE|wx.ALL, 5)
                
        sizer.Add(grid, 0, wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        # Register (button)....
        button = wx.Button(self, -1, "Register")
        self.Bind(wx.EVT_BUTTON, self.onRegister, button)
        
        buttonSizer = wx.StdDialogButtonSizer()
        buttonSizer.AddButton(wx.Button(self, wx.ID_OK))
        buttonSizer.AddButton(wx.Button(self, wx.ID_CANCEL))
        buttonSizer.Realize()
        buttonSizer.Insert(0, button)

        sizer.Add(buttonSizer, 0,
                  wx.GROW|wx.ALIGN_CENTER_VERTICAL|wx.ALL, 5)

        self.SetSizer(sizer)
        self.SetAutoLayout(True)
        sizer.Fit(self)

    def onRegister(self, evt):

        webbrowser.open(LICENSE_URL)

    def getParameters(self):

        return { 
            'license': self.licenseText.GetValue(),
        }


def promptLicense():

    dialog = LicenseDialog(wx.GetApp().mainFrame, -1)
    dialog.CenterOnScreen()

    if dialog.ShowModal() == wx.ID_OK:
        params = dialog.getParameters()
    else:
        params = None

    dialog.Destroy()

    if params is not None:
        license = params['license']
        if license:
            prefs = Utility.loadPrefs(Globals.options)
            pluginPrefs = prefs.setdefault(PLUGIN_NAME, {})
            pluginPrefs['license'] = license
            prefs.write()
            setLicense(license)
            return True

    return False



#   Copyright (c) 2003-2007 Open Source Applications Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

__parcel__ = "amazon"

import amazon, sgmllib, wx, string

from osaf.pim import ContentItem, ListCollection
from osaf.pim.types import LocalizableString
from repository.util.URL import URL
from application import schema
from i18n import MessageFactory
import AmazonDialog

_ = MessageFactory("Chandler-AmazonPlugin")


def _isEmpty(text):
    return not text or not text.strip()

def _showError(errText):
    theApp = wx.GetApp()
    if theApp is not None:
        wx.MessageBox(errText, _(u"Amazon Error"), parent=theApp.mainFrame)

def SearchByKeyword(repView, keywords=None, countryCode=None, category=None):
    """
    Performs an amazon search by keyword and creates an AmazonCollection
    containing AmazonItem's for each product found that matches the search
    criteria (only retrieves the first 10 products). If an AmazonCollection
    already exists for the search criteria the method creates AmazonItem's
    for new products and adds them to the existing collection.

    The method can be used programatically or via user input. If no keywords,
    countryCode, and category variables are passed in, an Amazon Search By
    Keyword dialog is displayed for the user to choose the keywords,
    countryCode, and category.

    The method contacts the Amazon site specified by the countryCode and
    retrieves the products that match the search criteria.

    @type repView: A Repository.view
    @param repView: The repository view in which to create the AmazonItems'
                    and AmazonCollection

    @type keywords: unicode
    @param keywords: The keywords to search on. If the value is None a
                     dialog is displayed for the user to enter the information.

    @type countryCode: unicode
    @param countryCode: The countryCode of the amazon site to contact.
                        If the value is None a dialog is displayed for
                        the user to enter the information.

    @type category: unicode
    @param category: The category to search in. If the value is None a
                     dialog is displayed for the user to enter the information.

    @rtype: AmazonCollection or None
    @return: An AmazonCollection containing AmazonItem's for each product or
             None if no products found or an error occurs.
    """

    if keywords is None or countryCode is None or category is None:
        keywords, countryCode, category = AmazonDialog.promptKeywords()

    if _isEmpty(keywords):
        """
        The user did not enter any text to search on or hit the cancel button
        """
        return None

    while True:
        try:
            bags = amazon.searchByKeyword(keywords, locale=countryCode,
                                          product_line=category)
            return _AddToCollection(repView, keywords, countryCode, bags) 
        except amazon.NoLicenseKey:
            if AmazonDialog.promptLicense():
                continue
            return None
        except (amazon.AmazonError, AttributeError), e:
            dt = {'keywords': keywords}
            _showError(_(u"No Amazon products were found for keywords '%(keywords)s'") % dt)
            return None


def SearchWishListByEmail(repView, emailAddr=None, countryCode=None):
    """
    Retrieves an amazon wishlist by email address and creates an
    AmazonCollection containing AmazonItem's for each product in
    the wishlist (only retrieves first 10 products found). If an
    AmazonCollection already exists for the search criteria the
    method creates AmazonItem's for new products and adds them to
    the existing collection.

    The method can be used programatically or via user input. If no
    emailAddr and countryCode variables are passed in, an Amazon
    WishList By Email dialog is displayed for the user to input the
    emailAddr and countryCode.

    The method contacts the Amazon site specified by the countryCode
    and retrieves the products for the wishlist.

    @type repView: A Repository.view
    @param repView: The repository view in which to create the AmazonItems'
                    and AmazonCollection

    @type emailAddr: unicode
    @param emailAddr: The email address for the user wishlist

    @type countryCode: unicode
    @param countryCode: The countryCode of the amazon site to contact.
                        If the value is None a dialog is displayed for
                        the user to enter the information.

    @rtype: AmazonCollection or None
    @return: An AmazonCollection containing AmazonItem's for each product or
             None if no products found or an error occurs.
    """

    if emailAddr is None or countryCode is None:
        emailAddr, countryCode = AmazonDialog.promptEmail()

    if _isEmpty(emailAddr):
        return None

    while True:
        try:
            customerName, bags = \
                amazon.searchWishListByEmail(emailAddr, locale=countryCode)
            return _AddToCollection(repView, customerName, countryCode, bags) 
        except amazon.NoLicenseKey:
            if AmazonDialog.promptLicense():
                continue
            return None
        except (amazon.AmazonError, AttributeError), e:
            dt = {'emailAddress': emailAddr}
            _showError(_(u"No Amazon Wishlist was found for email address '%(emailAddress)s'") % dt)
            return None


def _AddToCollection(repView, text, countryCode, bags):
    col, d = AmazonCollection.getCollection(repView, text, countryCode)

    counter = 0

    for aBag in bags:
        #_printBag(aBag, 0)
        #print "\n----------------------\n\n"

        #XXX This is temp for .6.
        # A dict of unique Amazon URLs is
        # returned with the collection.
        # The dict is searched and if an item with
        # the same URL exists a new item is not created.
        if not d.has_key(str(aBag.URL)):
            counter += 1
            col.add(AmazonItem(bag=aBag, itsView=repView))

    d = {'collectionName': col.displayName, 'numOf': counter}

    theApp = wx.GetApp()
    if theApp is not None:
        if counter == 0:
            msg = _(u"No new products were found for collection '%(collectionName)s'") % d
        elif counter == 1:
            msg = _(u"Added 1 product to collection '%(collectionName)s'") % d
        else:
            msg = _(u"Added %(numOf)s products to collection '%(collectionName)s'") % d
        theApp.CallItemMethodAsync("MainView", 'setStatusMessage', msg)

    repView.commit()

    return col


class AmazonCollection(ListCollection):
    keywords = schema.One(schema.Text)

    @classmethod
    def getCollection(cls, repView, text, countryCode):
        """
        Returns an AmazonCollection with a displayName combining the text
        and country code variables.

        The method checks to see if a AmazonCollection already exists
        matching the displayName combination of the text and countryCode
        variables. If one exists that collection is returned otherwise
        an AmazonCollection is created with the text / countryCode
        displayName and is returned.

        @type repView: A Repository.view
        @param repView: The repository view in which to create the AmazonItems'
                        and AmazonCollection

        @type text: unicode
        @param text: The text to use in combination with the countryCode to
                     create the AmazonCollection displayName

        @type countryCode: unicode
        @param countryCode: The countryCode to use in combination with the text
                            to create the AmazonCollection displayName

        @rtype: AmazonCollection
        @return: An existing AmazonCollection that matches the text/CountryCode
                 displayName combination or a new AmazonCollection if no
                 match is found.
       """


        displayName = AmazonCollection.makeCollectionName(text, countryCode)
        sidebarCollection = schema.ns("osaf.app", wx.GetApp().UIRepositoryView).sidebarCollection

        for collection in sidebarCollection:
            if collection.displayName.lower() == displayName.lower():
                #XXX: Create a dict of the productURL's of all items
                # of AmazonItem kind. If a url in the dict matches a 
                # new url that is downloaded it is not added to the 
                # collection

                d = {}
                for item in collection:
                    if isinstance(item, AmazonItem):
                        d[str(item.ProductURL)] = ''

                return collection, d

        collection = AmazonCollection(itsView=repView)
        collection.displayName = displayName
        sidebarCollection.add (collection)

        return collection, {}


    @classmethod
    def makeCollectionName(cls, text, countryCode):
        """
        Returns the displayName for an AmazonCollection based on the text
        and countryCode passed in.

        @type text: unicode
        @param text: The text to use in combination with the countryCode
                     to create the AmazonCollection displayName

        @type countryCode: unicode
        @param countryCode: The countryCode to use in combination with the
                            text to create the AmazonCollection displayName

        @rtype: unicode
        @return: The unicode displayName to be used for the AmazonCollection
        """

        if countryCode == 'gb':
            #The country code for the United Kingdom is 'GB' but
            #the web domain for the United Kingdom is .uk
            return u'Amzn.uk:' + text
        elif countryCode != 'us':
            return u'Amzn.' + countryCode + ': ' + text
        else:
            return u'Amzn:' + text

class AmazonItem(ContentItem):

    amazonCollection = schema.One(AmazonCollection)

    # When you add/remove/modify attributes here remember to update the
    # corresponding names displayed in the user interface in __init__.py
    ProductDescription = schema.One(schema.Text, indexed=True)
    Author = schema.One(schema.Text, indexed=True)
    Media = schema.One(schema.Text)
    ReleaseDate = schema.One(schema.Text)
    ImageURL = schema.One(schema.URL)
    ProductURL = schema.One(schema.URL)
    NewPrice = schema.One(schema.Text)
    UsedPrice = schema.One(schema.Text)
    Availability = schema.One(schema.Text)
    Manufacturer = schema.One(schema.Text)
    AverageCustomerRating = schema.One(schema.Text)
    NumberOfReviews = schema.One(schema.Text)
    
    @apply
    def ProductName():
        def fget(self):
            return self.displayName
        def fset(self, value):
            self.displayName = value
        return property(fget, fset)
 

    def _setBag(self, bag):
        self.ProductName = bag.ProductName
        desc = getattr(bag, 'ProductDescription', '')

        if desc != '':
            # The HTML is stripped because some descriptions
            # contain img src links to images as part of the description.
            # In theory this would be a nice feature but since the wx
            # HTML widget is slow and does not render till everything is
            # downloaded it impacts the display of the product considerably.
            desc = _stripHTML(desc)

        self.ProductDescription = desc
        self.Media = getattr(bag, 'Media', '')
        self.Manufacturer = getattr(bag, 'Manufacturer', '')
        self.NewPrice = getattr(bag, 'OurPrice', '')
        self.UsedPrice = getattr(bag, 'UsedPrice', '')
        self.Availability = getattr(bag, 'Availability', '')
        self.ImageURL = URL(bag.ImageUrlMedium.encode('ascii'))
        self.ProductURL = URL(bag.URL.encode('ascii'))
        self.ReleaseDate = getattr(bag, 'ReleaseDate','')

        if hasattr(bag, 'Authors'):
            if type(bag.Authors.Author) == type([]):
                self.Author = u', '.join(bag.Authors.Author)
            else:
                self.Author = bag.Authors.Author
        elif hasattr(bag, 'Directors'):
            if type(bag.Directors.Director) == type([]):
                self.Author = u', '.join(bag.Directors.Director)
            else:
                self.Author = bag.Directors.Director
        elif hasattr(bag, 'Artists'):
            if type(bag.Artists.Artist) == type([]):
                self.Author = ', '.join(bag.Artists.Artist)
            else:
                self.Author = bag.Artists.Artist
        else:
            # If no artist, author, or directory use the Manufacturer which
            # will either have a value or be '' by default
            self.Author = self.Manufacturer

        if hasattr(bag, 'Reviews'):
            self.AverageCustomerRating = getattr(bag.Reviews,
                                                 'AvgCustomerRating', '')
            self.NumberOfReviews = getattr(bag.Reviews,
                                           'TotalCustomerReviews', '')
        else:
            self.AverageCustomerRating = ''
            self.NumberOfReviews = ''

    bag = property(None, _setBag)


class DisplayNamesItem(schema.Item):
    namesDictionary = schema.Mapping(LocalizableString, defaultValue={})


def _printBag(aBag, level):
    """
    This is used for debugging the incoming Amazon XML which is
    parsed by amazon.py in to c{amazon.Bags}
    """
    for at in dir(aBag):
        val = getattr(aBag, at)

        if isinstance(val, unicode):
            val = val.encode('utf8')

        print "%s%s: %s" % ('\t'*level, at, val)

        if isinstance(val, amazon.Bag):
            _printBag(val, level+1)
        elif (isinstance(val, list) and len(val) > 0 and
              isinstance(val[0], amazon.Bag)):
            for bag in val:
                _printBag(bag, level+1)


class _Cleaner(sgmllib.SGMLParser):
    entitydefs={"nbsp": " "} 

    def __init__(self):
        sgmllib.SGMLParser.__init__(self)
        self.result = []
    def do_p(self, *junk):
        self.result.append(u'<p>')
    def do_br(self, *junk):
        self.result.append(u'<br>')
    def handle_data(self, data):
        self.result.append(data)
    def cleaned_text(self):
        txt = u''
        for uniText in self.result:
            if isinstance(uniText, str):
                uniText = unicode(uniText, 'utf8', 'ignore')

            txt += uniText
        return txt


def _stripHTML(text):
    c = _Cleaner()
    try:
        c.feed(text)
    except sgmllib.SGMLParseError:
        return text
    else:
        return c.cleaned_text()



#   Copyright (c) 2003-2007 Open Source Applications Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

from datetime import timedelta

from AmazonKinds import AmazonCollection, AmazonItem, DisplayNamesItem
from AmazonBlocks import AmazonController, AmazonDetailBlock
from AmazonDialog import LicenseTask

from application import schema
from osaf.views.detail import makeSubtree
from osaf.pim.structs import SizeType, RectType
from i18n import MessageFactory

_ = MessageFactory("Chandler-AmazonPlugin")


def installParcel(parcel, version=None):

    controller = AmazonController.update(parcel, "controller")

    blocks = schema.ns('osaf.framework.blocks', parcel)
    main   = schema.ns('osaf.views.main', parcel)
    detail = schema.ns('osaf.views.detail', parcel)

    amazonMenu = blocks.Menu.update(parcel, 'AmazonDemoMenu',
                                    blockName = 'AmazonDemoMenu',
                                    title = _(u'Amazon'),
                                    helpString = _(u'Download wishlists from Amazon'),
                                    childrenBlocks = [ ],
                                    parentBlock = main.ExperimentalMenu)

    blocks.MenuItem.update(parcel, "NewAmazonCollection",
        blockName = "NewAmazonCollectionMenu",
        title = _(u"Amazon &Keyword Search..."),
        event = blocks.BlockEvent.update(parcel, "NewAmazonCollectionEvent",
            blockName = "NewAmazonCollection",
            dispatchEnum = "SendToBlockByReference",
            destinationBlockReference = controller,
            commitAfterDispatch = True,
        ),
        eventsForNamedLookup = [parcel["NewAmazonCollectionEvent"]],
        parentBlock = amazonMenu,
    )

    blocks.MenuItem.update(parcel, "NewAmazonWishList",
        blockName = "NewAmazonWishListMenu",
        title = _(u"Amazon &Wish List Search..."),
        event = blocks.BlockEvent.update(parcel, "NewAmazonWishListEvent",
            blockName = "NewAmazonWishList",
            dispatchEnum = "SendToBlockByReference",
            destinationBlockReference = controller,
            commitAfterDispatch = True,
        ),
        eventsForNamedLookup = [parcel["NewAmazonWishListEvent"]],
        parentBlock = amazonMenu,
    )

    makeSubtree(parcel, AmazonItem, [
        detail.MarkupBar,
        AmazonDetailBlock.update(parcel, "amazonDetail",
            blockName = "amazonDetail",
            size = SizeType(100,50),
            minimumSize = SizeType(100,50),
        ),
    ])

    DisplayNamesItem.update(parcel, "displayNames",
        namesDictionary = {'ProductName': _(u'Product Name'),
                           'ProductDescription': _(u'Product Description'),
                           'Author': _(u'Author(s)'),
                           'Media': _(u'Media'),
                           'ReleaseDate': _(u'Release Date'),
                           'ImageURL': _(u'image path'),
                           'ProductURL': _(u'product url'),
                           'NewPrice': _(u'New Price'),
                           'UsedPrice': _(u'Used Price'),
                           'Availability': _(u'Availability'),
                           'Manufacturer': _(u'Manufacturer'),
                           'AverageCustomerRating': _(u'Average Customer Review'),
                           'NumberOfReviews': _(u'Number of people who reviewed the item')})

    LicenseTask.update(parcel, "licenseTask",
                       run_at_startup=True,
                       interval=timedelta(days=1))



#   Copyright (c) 2003-2007 Open Source Applications Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


__parcel__ = "amazon"

from application import schema
import AmazonKinds
import osaf.framework.blocks.Block as Block
import osaf.views.detail as Detail
from i18n import MessageFactory

_ = MessageFactory("Chandler-AmazonPlugin")

class AmazonController(Block.Block):
    def onNewAmazonCollectionEvent(self, event):
        """
        Called when a NewAmazonCollection Search by Keyword Event is triggered.

        @type event: BlockEvent
        @param event: The BlockEvent triggered on NewAmazonCollection command

        @rtype: AmazonCollection or C{None}
        @return: AmazonCollection for the search keyword or C{None}
                 if search results == 0  
        """
        return AmazonKinds.SearchByKeyword(self.itsView)

    def onNewAmazonWishListEvent(self, event):
        """
        Called when a NewAmazonCollection Email Wishlist Event is triggered.

        @type event: BlockEvent
        @param event: The BlockEvent triggered on NewAmzonWishListEvent command

        @rtype: AmazonCollection or C{None}
        @return: AmazonCollection for the wishlist or None
                 if search results == 0
        """
        return AmazonKinds.SearchWishListByEmail(self.itsView)


class AmazonDetailBlock(Detail.HTMLDetailArea):
    """
    HTMLDetailArea block which renders an AmazonItem as an HTML page
    including the Product picture and customer rating.
    """

    ROW_FONT = "<font face='arial, verdana, helvetica' color='black' size='%s'>%s</font>"
    STAR_URL = "http://g-images.amazon.com/images/G/01/x-locale/common/customer-reviews/stars-%s-%s.gif"

    def getHTMLText(self, item):
        """
        Generates the HTML layout of an AmazonItem.

        @type item: AmazonItem
        @param item: The AmazonItem to use for the HTMLDetailArea

        @rtype: unicode
        @return: The HTML layout for the AmazonItem
        """
        if item is None or item == item.itsView:
            return None

        HTMLText = u'<html><head><body>\n'
        HTMLText += "<b>" + self._applyFont(item.ProductName, 3) + "</b><p>\n"
        HTMLText += "<table cellspacing=2 cellpadding=2 border=0>\n"
        HTMLText += "<tr><td width='30%' valign='top' align='center'><a href='" + str(item.ProductURL) + "'><img src='" + str(item.ImageURL) + "' border=1><br>" + self._applyFont(_(u"More Product Details"), 1) + "</a></td>\n"
        HTMLText += "<td valign='top' align='left'><table width='100%' border=0 cellspacing=2 cellpadding=2>"

        HTMLText += self._makeRow(item, 'Author')
        HTMLText += self._makeRow(item, 'NewPrice')
        HTMLText += self._makeRow(item, 'UsedPrice')
        HTMLText += self._makeRow(item, 'Availability')
        HTMLText += self._makeRow(item, 'ReleaseDate')
        HTMLText += self._makeRating(item)
        HTMLText += self._makeRow(item, 'Manufacturer')
        HTMLText += self._makeRow(item, 'Media')
        HTMLText += "</table>"

        if item.ProductDescription != "":
            HTMLText += "<tr><td colspan=2><br><hr>" + item.ProductDescription + "</td></tr>"

        HTMLText += "</td></tr>\n"
        HTMLText += "</table></body></html>"

        return HTMLText

    def _makeRow(self, item, field):
        val = getattr(item, field, '')
        if val == '':
            return ''

        displayNamesItem = schema.ns(__name__, self.itsView).displayNames
        displayName = displayNamesItem.namesDictionary [field]

        return u"<tr><td align='right' valign='top' width='40%'>" + self._applyFont("<b>" + displayName + ":</b>") + \
               "</td><td align='left' valign='top'>" + self._applyFont(val) + "</td></tr>"

    def _makeRating(self, item):
        if item.AverageCustomerRating == '':
            return ''

        displayNamesItem = schema.ns(__name__, self.itsView).displayNames
        displayName = displayNamesItem.namesDictionary ["AverageCustomerRating"]

        txt = u"<tr><td align='right' valign='top' width='40%'>" + self._applyFont("<b>" + displayName + ":</b>") + "</td>"

        txt += "<td align='left' valign='top'><img src='" + self._getRatingURL(item.AverageCustomerRating) + "'>"
        txt += self._getNumReviews(item.NumberOfReviews)

        txt += "</td></tr>"
        return txt

    def _getNumReviews(self, reviews):
        if reviews == '':
            return ''

        txt = "&nbsp;("

        if int(reviews) == 1:
            txt += _(u"1 customer review")
        else:
            txt += _(u"%(numberOf)s customer reviews") % {'numberOf': reviews}

        txt += ")"

        return self._applyFont(txt)

    def _getRatingURL(self, rating):
        val = rating.split(".")
        numOne = val[0]
        numTwo = 0

        if len(val) > 1 and int(val[1]) >= 5:
            numTwo = 5

        return self.STAR_URL % (numOne, numTwo)

    def _applyFont(self, val, size=2):
        return self.ROW_FONT % (size, val)



# -*- coding: utf-8 -*-
#   Copyright (c) 2004-2007 Open Source Applications Foundation
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.


""" Unit test for amazon i18n I/O """

import unittest
from osaf.pim.tests import TestDomainModel
from amazon import AmazonKinds
import os

#XXX: The unit test has been disabled for tinderbox builds till
#     a mock server can be put in place.
#     The test can fail on the Amazon server side
#     which can lead to false positives.
#     The test will still run using the run_tests framework
if False:
    class TestI18nAmazon(TestDomainModel.DomainModelTestCase):
    
        def setUp(self):
            super(TestI18nAmazon, self)._setup(self)
    
            self.testdir = os.path.join(self.rootdir, 'parcels', 'amazon')
    
            super(TestI18nAmazon, self)._openRepository(self)
            self.loadParcel("amazon")
    
        def testI18nSearch(self):
            if not self.isOnline():
                return
    
            #Try and retrieve a keyword which is not valid. This tests that the
            #parcel correctly handles the case when data is not returned
            col = AmazonKinds.SearchByKeyword(self.view, self.view, u'sdsdsdasdasdasdsDFDSFSDFsdasd', 'us', 'music')
            self.assertEquals(col, None)
    
            #Test that the Japanese product name is returned by an amazon search for
            #keyword 'Rap' in the Music category of amazon.co.jp
            productName = u'RAPâ€•ã�“ã‚Œã�§ãƒ©ãƒƒãƒ—ãƒ»ãƒŸãƒ¥ãƒ¼ã‚¸ãƒƒã‚¯ã�Œã‚�ã�‹ã‚‹'
            col = AmazonKinds.SearchByKeyword(self.view, self.view, 'Rap', 'jp', 'music')
            self.assertNotEquals(col, None)
            self.assertTrue(self.hasKey(col, productName))
    
            #Test that the German product name is return by an amazon search for
            #keyword == productName in the Music category of amazon.de
            productName = u'Im ZauberschloÃŸ... Auf dem Weg zum Schlafen und TrÃ¤umen'
            col = AmazonKinds.SearchByKeyword(self.view, self.view, 'Zauberschloss', 'de', 'music')
            self.assertNotEquals(col, None)
            self.assertTrue(self.hasKey(col, productName))
    
    
        def testI18nEmail(self):
            if not self.isOnline():
                return
    
            #A wishlist for bkirsch@osafoundation.org on amazon.com contains theses
            #four non-ascii product names. Retrieve the list from amazon.com and confirm
            #that these titles are in the list
            productNames = [
                            u"Die Ãœbungspatrone: HÃ¶rspiele (Literatur heute)",
                            u"FrÃ¼he sozialistische HÃ¶rspiele",
                            u"El lÃ¡piz del carpintero (Punto de Lectura)",
                            u"Cien AÃ±os de Soledad"
                           ]
    
            col = AmazonKinds.SearchWishListByEmail(self.view, self.view, \
                                                     'bkirsch@osafoundation.org', 'us')
    
            self.assertNotEquals(col, None)
    
            for productName in productNames:
                self.assertTrue(self.hasKey(col, productName))
    
            #This email address does not have an amazon.com wishlist. This tests
            #that the parcel correctly handles the case when data is not returned
            col = AmazonKinds.SearchWishListByEmail(self.view, self.view, \
                                                     'fake_address@osafoundation.org', 'us')
    
            self.assertEquals(col, None)
    
    
        def hasKey(self, col, key):
            if col is None:
                return False
    
            for item in col:
                if item.displayName == key:
                    return True
            return False


if __name__ == "__main__":
    unittest.main()
 
